/*
 * Name: kstat knetstat.c
 * Date: Sun Feb 10 02:49:49 CET 2002
 * Author: FuSyS [fusys@s0ftpj.org, fusys@sikurezza.org]
 *
 * SoftProject Digital Security for Y2K
 * Sikurezza.org - Italian Security Mailinglist
 *
 * MOJITO-WARE LICENSE - This source code is like "THE BEER-WARE LICENSE" by
 * Poul-Henning Kamp <phk@FreeBSD.ORG> but you can give me in return a mojito.
 *
 * Tested on: Slackware 8 Linux MaNTRa 2.4.16 #4 i686 unknown
 */

#include "linux.h"


struct net_task {
	pid_t pid;
	int lastcfd;
};

char sname[16];

void lookproto(unsigned char state)
{
        switch(state) {
                case 1:
                        printf("%15s", "ESTABLISHED ");
                        break;
                case 2:
                        printf("%16s", "SYN_SENT ");
                        break;
                case 3:
                        printf("%16s", "SYN_RECV ");
                        break;
                case 4:
                        printf("%16s", "FIN_WAIT1 ");
                        break;
                case 5:
                        printf("%16s", "FIN_WAIT2 ");
                        break;
                case 6:
                        printf("%16s", "TIME_WAIT ");
                        break;
                case 7:
                        printf("%16s", "CLOSE ");
                        break;
                case 8:
                        printf("%16s", "CLOSE_WAIT ");
                        break;
                case 9:
                        printf("%16s", "LAST_ACK ");
                        break;
                case 10:
                        printf("%16s", "LISTEN ");
                        break;
                case 11:
                        printf("%16s", "CLOSING ");
                        break;
                default:
                        break;
         }
}

#ifdef IPV6
void parse_sock6(struct in6_addr saddr, unsigned short sport, struct in6_addr daddr, unsigned short dport,
unsigned char state, unsigned char protocol, unsigned short type, pid_t pid, int fd)
{
	char *sad, *dad;

	switch(protocol) {
                case 6:
                        printf("%-5s", "tcp6");
                        break;
                case 17:
                        printf("%-5s", "udp6");
                        break;
                case 255:
                        printf("%-5s", "raw6");
                default:
                        break;
        }
        sad=ntoa6(saddr);
        printf("%18s:%-5d ", sad, sport);
        dad=ntoa6(daddr);
        printf("%18s:", dad);
	if(!dport) printf("*    "); else printf("%-5d ", dport);
	if(protocol == 6) lookproto(state); else printf("%16s", "");
	printf("%17s [%5s:%-6d %4s:%-4d]\n", sname, "Pid", pid, "Fd", fd);
}
#endif

void parse_sock4(unsigned long saddr, unsigned short sport, unsigned long daddr, unsigned short dport, 
unsigned char state, unsigned char protocol, unsigned short type, pid_t pid, int fd)
{
	char *sad, *dad;

	switch(protocol) {
		case 6:
			printf("%-5s", "tcp");
			break;
		case 17:
			printf("%-5s", "udp");
			break;
		case 255:
			printf("%-5s", "raw");
		default:
			break;
	}
	sad=ntoa(saddr); 
	printf("%18s:%-5d ", sad, sport);
	dad=ntoa(daddr);
	printf("%18s:", dad);
	if(!dport) printf("*    "); else printf("%-5d ", dport);
	if(protocol == 6) lookproto(state); else printf("%16s", "");
	printf("%17s [%5s:%-6d %4s:%-4d]\n", sname, "Pid", pid, "Fd", fd);
}

void show_netstat()
{
	unsigned long addr, swapper;
	struct task_struct task, taskk;
	struct net_task ps[NR_TASKS]; 
	struct files_struct files_ptr;
	struct file *f, fd;
	struct dentry dentry;
	struct inode inode;
	struct sock sk;
	umode_t type;  
	unsigned int kstat_addr=0; 
	int kd, i, lastpid=0, cfd;

        for(i=1; i < NR_TASKS; i++) ps[i].pid=0; 

	kstat_addr=find_kmem_offset(TASK_OFF);

	kd=open(KMEM, O_RDONLY);
	if(kread(kd, (unsigned long)kstat_addr, &task, sizeof(struct task_struct)) == -1) err("kread error");
	addr=swapper=(unsigned long)&task;

     	printf("\nActive Internet connections (servers and established)\n\n");
  
	for(i=1; i < NR_TASKS; i++){
		if(kread(kd, (unsigned long)addr, &taskk, sizeof(struct task_struct)) == -1) err("kread error");
		if(kread(kd,(unsigned long)taskk.comm, &sname, 15) == -1) err("kread error"); sname[16]='\0';
		if(taskk.files) {
		  if(kread(kd, (unsigned long)taskk.files, &files_ptr, sizeof(struct files_struct)) == -1) err("kread error1");
		  for(cfd=0,f=&fd;cfd<files_ptr.max_fds;cfd++){
			if(kread(kd, (unsigned long)files_ptr.fd++, &f, sizeof(fd))== -1) err("kread error2");
		  	if(f){ 
			   if(kread(kd, (unsigned long)f, &fd, sizeof(struct file)) == -1) err("kread error3");
		  	  if(fd.f_dentry)
			   if(kread(kd, (unsigned long)fd.f_dentry, &dentry, sizeof(struct dentry)) == -1) err("kread error4");
			  if(dentry.d_inode)
			   if(kread(kd, (unsigned long)dentry.d_inode, &inode, sizeof(struct inode)) == -1) err("kread error6");
		  	   type = inode.i_mode & S_IFMT;
		  	   switch(type){
		    	    case S_IFSOCK:
				lastpid=taskk.pid;
				if(kread(kd, (unsigned long)inode.u.socket_i.sk, &sk, sizeof(struct sock)) == -1) err("kread errorZ");
				if((!ps[lastpid].pid || cfd > ps[lastpid].lastcfd) && sk.family == 2){ 
					parse_sock4(sk.rcv_saddr, sk.num, sk.daddr, ntohs(sk.dport), sk.state,
						sk.protocol, sk.type, taskk.pid, cfd);
					ps[lastpid].pid=lastpid;
					ps[lastpid].lastcfd=cfd;
				}
#ifdef IPV6
				else if((!ps[lastpid].pid || cfd > ps[lastpid].lastcfd) && sk.family == 10){
					parse_sock6(sk.net_pinfo.af_inet6.rcv_saddr, sk.num,
						sk.net_pinfo.af_inet6.daddr, ntohs(sk.dport), sk.state,
						sk.protocol, sk.type, taskk.pid, cfd);
					ps[lastpid].pid=lastpid;
					ps[lastpid].lastcfd=cfd;
				}
#endif
		    		break;
		    	     default:
				break;
		  	    }
	  		}
	       	   }
		}
	  addr=(unsigned long)taskk.next_task;
	}
	printf("\n");
}
