/*
 *  Copyright (C) 2000 Yoann Vandoorselaere.
 *
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Authors: Yoann Vandoorselaere <yoann@mandrakesoft.com>
 *
 */

/*
 * Include this header for portability.
 */

#define __USE_BSD 1
#define __FAVOR_BSD 1

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/in_systm.h>
#include <netinet/ip.h>
#define __FAVOR_BSD 1
#include <netinet/tcp.h>
#include <netinet/udp.h>
#include <netinet/igmp.h>
#include <net/if.h>



#define ETHERTYPE_IP      0x0800
#define ETHERTYPE_ARP     0x0806 /* Address resolution */
#define ETHERTYPE_REVARP  0x8035 /* Reverse ARP */

#define ETH_ALEN  6


struct arphdr {
        uint16_t ar_hrd;     /* format of hardware address */
        uint16_t ar_pro;     /* format of protocol address */
        uint8_t ar_hln;     /* length of hardware address */
        uint8_t ar_pln;     /* length of protocol address */
        uint16_t ar_op;      /* one of: */
};



struct ether_arp {
        struct arphdr ea_hdr;
        
        uint8_t arp_sha[ETH_ALEN]; /* sender hardware address */
        uint8_t arp_spa[4];        /* sender protocol address */
        uint8_t arp_tha[ETH_ALEN]; /* target hardware address */
        uint8_t arp_tpa[4];        /* target protocol address */

};



#define arp_hrd ea_hdr.ar_hrd
#define arp_pro ea_hdr.ar_pro
#define arp_hln ea_hdr.ar_hln
#define arp_pln ea_hdr.ar_pln
#define arp_op  ea_hdr.ar_op


/*
 * Structure of a DEC/Intel/Xerox or 802.3 Ethernet header.
 */
struct ether_header {
    uint8_t    ether_dhost[ETH_ALEN];
    uint8_t    ether_shost[ETH_ALEN];
    uint16_t   ether_type;
};




/*
 * FDDI header
 */
struct fddi_header {
        uint8_t fddi_fc;          /* Frame Control (FC) value */
        uint8_t fddi_dhost[6];    /* Destination host */
        uint8_t fddi_shost[6];    /* Source host */
};


#define FDDI_HDRLEN 13





/*
 * Internal of an ICMP Router Advertisement
 */

#define ICMP_MINLEN 8
#define ICMP_DEST_UNREACH 3



struct icmp_ra_addr {
        uint32_t ira_addr;
        uint32_t ira_preference;
};


struct icmp {
        uint8_t  icmp_type;  /* type of message, see below */
        uint8_t  icmp_code;  /* type sub code */
        uint16_t icmp_cksum; /* ones complement checksum of struct */

        union {
                unsigned char ih_pptr;       /* ICMP_PARAMPROB */
                struct in_addr ih_gwaddr;   /* gateway address */

                struct ih_idseq {           /* echo datagram */
                        uint16_t icd_id;
                        uint16_t icd_seq;
                } ih_idseq;

                uint32_t ih_void;

                /* ICMP_UNREACH_NEEDFRAG -- Path MTU Discovery (RFC1191) */
                struct ih_pmtu {
                        uint16_t ipm_void;
                        uint16_t ipm_nextmtu;
                } ih_pmtu;
                 
                struct ih_rtradv {
                        uint8_t irt_num_addrs;
                        uint8_t irt_wpa;
                        uint16_t irt_lifetime;
                } ih_rtradv;
        } icmp_hun;
        
#define icmp_pptr       icmp_hun.ih_pptr
#define icmp_gwaddr     icmp_hun.ih_gwaddr
#define icmp_id         icmp_hun.ih_idseq.icd_id
#define icmp_seq        icmp_hun.ih_idseq.icd_seq
#define icmp_void       icmp_hun.ih_void
#define icmp_pmvoid     icmp_hun.ih_pmtu.ipm_void
#define icmp_nextmtu    icmp_hun.ih_pmtu.ipm_nextmtu
#define icmp_num_addrs  icmp_hun.ih_rtradv.irt_num_addrs
#define icmp_wpa        icmp_hun.ih_rtradv.irt_wpa
#define icmp_lifetime   icmp_hun.ih_rtradv.irt_lifetime
        union {
                struct {
                        uint32_t its_otime;
                        uint32_t its_rtime;
                        uint32_t its_ttime;
                } id_ts;
                
                struct {
                        struct ip idi_ip;  
                        unsigned char data[8];
                } id_ip;

                struct icmp_ra_addr id_radv;
                uint32_t   id_mask;
                uint8_t    id_data[1];
        } icmp_dun;

#define icmp_otime      icmp_dun.id_ts.its_otime
#define icmp_rtime      icmp_dun.id_ts.its_rtime
#define icmp_ttime      icmp_dun.id_ts.its_ttime
#define icmp_ip         icmp_dun.id_ip.idi_ip
#define icmp_radv       icmp_dun.id_radv
#define icmp_mask       icmp_dun.id_mask
#define icmp_data       icmp_dun.id_data
        
};
