#ifndef PACKET_H
#define PACKET_H

/*****************************************************************************
 * Prelude                                                                   *
 *                                                                           *
 * (C) 1998 - 2000, Vandoorselaere Yoann                                     *
 *****************************************************************************/

/*****
*
* Copyright (C) 1998 - 2000 Vandoorselaere Yoann
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include "proto.h"
#include "nethdr.h"


#define MAX_PKTDEPTH 20

/*
 * ip_hl is 4 bits long and is the number of 32 bits words
 * in the header including options :
 * so max ip_hl value is ((4 ^ 2) -1) = 15
 * 15 * 4 = 60. ( 4 is the number of bytes in one 32 bits word ).
 * 60 - 20 = 40 ( 20 == size of tcp / ip headers without options ).
 */
#define __MAX_OPTS_LEN 40

/*
 * Value from rfc 1072, 1323, 1644, 1693.
 */
#define __TCPOPT_EOL 0
#define __TCPOPT_NOP 1
#define __TCPOPT_MAXSEG 2
#define __TCPOPT_WSCALE 3
#define __TCPOPT_SACK_PERMITTED 4
#define __TCPOPT_SACK 5
#define __TCPOPT_ECHO 6
#define __TCPOPT_ECHOREPLY 7
#define __TCPOPT_TIMESTAMP 8
#define __TCPOPT_POC_PERMITTED 9
#define __TCPOPT_POC 10
#define __TCPOPT_CC 11
#define __TCPOPT_CCNEW 12
#define __TCPOPT_CCECHO 13

#define __TCPOLEN_EOL 1
#define __TCPOLEN_NOP 1
#define __TCPOLEN_MAXSEG 4
#define __TCPOLEN_WSCALE 3
#define __TCPOLEN_SACK_PERMITTED 2
#define __TCPOLEN_ECHO 6
#define __TCPOLEN_ECHOREPLY 6
#define __TCPOLEN_TIMESTAMP 10
#define __TCPOLEN_CC 6
#define __TCPOLEN_CCNEW 6
#define __TCPOLEN_CCECHO 6
#define __TCPOLEN_POC_PERMITTED 2
#define __TCPOLEN_POC 3

#define __IPOPT_EOL               0
#define __IPOPT_NOP               1
#define __IPOPT_RR                7
#define __IPOPT_TIMESTAMPS        68
#define __IPOPT_SECURITY          130
#define __IPOPT_LSRR              131
#define __IPOPT_SATID             136
#define __IPOPT_SSRR              137
#define __IPOPT_RA                148

/*
 * Packet stuff
 */
typedef struct
{
	size_t size;
        unsigned char data[0];
} PktData_t;


struct __iphdr
{
        struct ip ip_hdr;
        unsigned char opts[__MAX_OPTS_LEN];
        int opts_len;
};



struct __tcphdr 
{
        struct tcphdr tcp_hdr;
        unsigned char opts[__MAX_OPTS_LEN];
        int opts_len;
};



union proto_u {
        struct ether_header *ether_hdr;
        struct __iphdr *ip;
        struct __tcphdr *tcp;
        struct udphdr *udp_hdr;
        struct icmp *icmp_hdr;
        struct igmp *igmp_hdr;
        struct ether_arp *arp_hdr;
        PktData_t *pkt_data;
};



typedef struct {
        size_t len;
	proto_enum_t proto;
        int refcnt;
	union proto_u p;
} Packet_t;


#endif
