/*
 *  Copyright (C) 2000 Yoann Vandoorselaere.
 *
 *  This program is free software; you can redistribute it and/or modify 
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  Authors: Yoann Vandoorselaere <yoann@mandrakesoft.com>
 *
 */
#include <stdio.h>
#include <stdlib.h>

#include "common.h"
#include "packet.h"
#include "alert.h"
#include "alert-prv.h"
#include "alert-common.h"
#include "socket-op.h"


#define do_read(fd, ptr, size, mread) do {                    \
        if ( socket_read_nowait(fd, ptr, size, mread) <= 0)   \
               goto err;                                      \
} while(0)



/*
 * I would like OpenSSL to support readv()...
 * It would be a lot cleaner, and a lot faster...
 */
static int read_alert(int fd, alert_t *alert,
                      ssize_t (*my_read)(int fd, void *data, size_t size)) 
{
        int ret = 0;

        ret = socket_read(fd, &alert_from_len(alert), sizeof(alert_from_len(alert)), my_read);
        if ( ret <= 0 )
                goto err;
        
        do_read(fd, &alert_author_len(alert), sizeof(alert_author_len(alert)), my_read);
        do_read(fd, &alert_contact_len(alert), sizeof(alert_contact_len(alert)), my_read);
        do_read(fd, &alert_desc_len(alert), sizeof(alert_desc_len(alert)), my_read);
        do_read(fd, &alert_quickmsg_len(alert), sizeof(alert_quickmsg_len(alert)), my_read);
        do_read(fd, &alert_kind(alert), sizeof(alert_kind(alert)), my_read);
        do_read(fd, &alert_count(alert), sizeof(alert_count(alert)), my_read);
        do_read(fd, &alert_time_start(alert), sizeof(alert_time_start(alert)), my_read);
        do_read(fd, &alert_time_end(alert), sizeof(alert_time_end(alert)), my_read);
        do_read(fd, &alert_depth(alert), sizeof(alert_depth(alert)), my_read);
        do_read(fd, &alert_message_len(alert), sizeof(alert_message_len(alert)), my_read);

        /*
         * Message is a static array.
         */
        if ( alert_message_len(alert) > sizeof(alert_message(alert)) ) {
                log(LOG_ERR, "Buffer overflow attempt against Prelude : %d > %d.\n",
                    alert_message_len(alert), sizeof(alert_message(alert)));
                ret = -1;
                goto err;
        }
        
        do_read(fd, &alert_message(alert), alert_message_len(alert), my_read);
        
        alert_from(alert) = malloc(alert_from_len(alert) + 1);
        if ( ! alert_from(alert) )
                goto err;

        ret = socket_read_nowait(fd, alert_from(alert), alert_from_len(alert), my_read);
        if ( ret < 0 ) {
                free(alert_from(alert));
                goto err;
        }
        alert_from(alert)[alert_from_len(alert)] = '\0';
        
        alert_author(alert) = malloc(alert_author_len(alert) + 1);
        if (! alert_author(alert) ) {
                free(alert_from(alert));
                goto err;
        }

        ret = socket_read_nowait(fd, alert_author(alert), alert_author_len(alert), my_read);
        if ( ret <= 0 ) {
                free(alert_from(alert));
                free(alert_author(alert));
                goto err;
        }
        alert_author(alert)[alert_author_len(alert)] = '\0';
        

        alert_contact(alert) = malloc(alert_contact_len(alert) + 1);
        if ( ! alert_contact(alert) ) {
                free(alert_from(alert));
                free(alert_author(alert));
                goto err;
        }

        ret = socket_read_nowait(fd, alert_contact(alert), alert_contact_len(alert), my_read);
        if ( ret <= 0 ) {
                free(alert_from(alert));
                free(alert_author(alert));
                free(alert_contact(alert));
                goto err;
        }
        alert_contact(alert)[alert_contact_len(alert)] = '\0';
        

        alert_desc(alert) = malloc(alert_desc_len(alert) + 1);
        if ( ! alert_desc(alert) ) {
                free(alert_from(alert));
                free(alert_author(alert));
                free(alert_contact(alert));
                goto err;
        }

        ret = socket_read_nowait(fd, alert_desc(alert), alert_desc_len(alert), my_read);
        if ( ret <= 0 ) {
                free(alert_from(alert));
                free(alert_author(alert));
                free(alert_contact(alert));
                free(alert_desc(alert));
                goto err;
        }
        alert_desc(alert)[alert_desc_len(alert)] = '\0';
        
        
        alert_quickmsg(alert) = malloc(alert_quickmsg_len(alert) + 1);
        if ( ! alert_quickmsg(alert) ) {
                free(alert_from(alert));
                free(alert_author(alert));
                free(alert_contact(alert));
                free(alert_desc(alert));
                goto err;
        }

        ret = socket_read_nowait(fd, alert_quickmsg(alert), alert_quickmsg_len(alert), my_read);
        if ( ret <= 0 ) {
                free(alert_from(alert));
                free(alert_author(alert));
                free(alert_contact(alert));
                free(alert_desc(alert));
                free(alert_quickmsg(alert));
                goto err;
        }
        alert_quickmsg(alert)[alert_quickmsg_len(alert)] = '\0';

        
        return ret;

 err:
        if ( ret < 0 )
                log(LOG_ERR, "Error reading alert.\n");

        return ret;
}





int alert_read(int sock, alert_t *alert,
               ssize_t (*my_read)(int fd, void *data, size_t size)) 
{
        int i, ret;
        Packet_t *p = alert_packet(alert);
        
        ret = read_alert(sock, alert, my_read);
        if ( ret <= 0 )
                goto err;

        for ( i = 0;; i++ ) {
                do_read(sock, &p[i].len, sizeof(p[i].len), my_read);
                do_read(sock, &p[i].proto, sizeof(p[i].proto), my_read);
                
                if ( p[i].proto == p_end )
                        break;

                p[i].p.ip = malloc(p[i].len);
                if ( ! p[i].p.ip )
                        goto err;

                do_read(sock, p[i].p.ip, p[i].len, my_read);
        } 
        
        return 1;

 err:
        if ( ret < 0 )
                log(LOG_ERR, "Error reading report.\n");
        
        return ret;
}





/*
 *
 */
void alert_free(alert_t *alert, int pfree) 
{
        if ( alert_from(alert) )
                free(alert_from(alert));
        
        if ( alert_desc(alert) )
                free(alert_desc(alert));

        if ( alert_author(alert) )
                free(alert_author(alert));
        
        if ( alert_contact(alert) )
                free(alert_contact(alert));

        if ( alert_quickmsg(alert) )
                free(alert_quickmsg(alert));
        
        if ( pfree ) {
                int i;
                
                for ( i = 0; alert->packet[i].proto != p_end; i++ )
                        if ( alert->packet[i].p.ip )
                                free(alert->packet[i].p.ip);
        }
}





