/*****
*
* Copyright (C) 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>

#include "detect.h"


static plugin_detect_t plugin;
static subscribtion_t subscribtion[] = {
        { p_all, NULL },
        { p_end, NULL },
};



static void debug_run(Packet_t *packet, int depth) 
{
        plugin_do_report((plugin_generic_t *) &plugin, packet, normal,
                         "Debug reporting",
                         "This is a debuging message by the debug plugin");

}



static void print_help(void) 
{
        fprintf(stderr, "Usage for %s :\n", plugin.name);
        fprintf(stderr, "\t -e --enable-debug (default to disable)\n\n");
}



static int check_opts(void) 
{
        char **argv;
        config_t *cfg;
        int argc, c, enable = -1;
        struct option opts[] = {
                { "enable-debug", no_argument, NULL, 'e' },
                { "help", no_argument, NULL, 'h'         },
                { 0, 0, 0, 0 },
        };

        plugin_get_opts(plugin.name, &argc, &argv);
                
        while ( (c = getopt_long(argc, argv, "eh", opts, NULL)) != -1 ) {

                switch (c) {
                case 'e':
                        enable = 1;
                        break;

                case 'h':
                        print_help();
                        return -1;
                        
                default:
                        return -1;
                }
        }
        
        if ( ! enable && (cfg = config_open(PRELUDE_CONF)) ) {
                const char *str;
                
                str = config_get(cfg, plugin.name, "enable");
                if ( str && strcmp(str, "yes") == 0 )
                        enable = 1;
                
                config_close(cfg);
        }

        return enable;
}


int plugin_init(unsigned int id)
{
        plugin_set_name(&plugin, "Debug");
        
        if ( check_opts() < 0 )
                return -1;
        
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "Send an alert for each packet.");
        plugin_set_running_func(&plugin, debug_run);
        plugin_set_subscribtion(&plugin, subscribtion);
        
        return plugin_register((plugin_generic_t *) &plugin);
}
