/*****************************************************************************
 * Prelude                                                                   *
 *                                                                           *
 * (C) 1998 - 2000, Vandoorselaere Yoann                                     *
 *****************************************************************************/

/*****
*
* Copyright (C) 1998 - 2000 Vandoorselaere Yoann
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "detect.h"

#define MAX_NOP 600

static plugin_detect_t plugin;
static subscribtion_t subscribtion[] = {
        { p_data, NULL },
	{ p_end,  NULL }
};



/*
 * When the prelude rules engine will be powerfull enough,
 * this plugins will not exist anymore.
 */

static int alloc_bstring(unsigned char **bstring, size_t *bsize, unsigned char *nop, int nopsize) 
{
        int i;
        size_t size;
        unsigned char *s;
                
        *bsize = size = MAX_NOP * nopsize;        
        *bstring = malloc(size);
        
        if (! *bstring ) {
                log(LOG_ERR, "malloc(%d)", size);
                return -1;
        }

        s = *bstring;
        
        for ( i = 0; i < size; i += nopsize )
                memcpy(&s[i], nop, nopsize);

        return 0;
}

static unsigned char *x86_bstring, *alpha_bstring, *ppc_bstring, *sparc_bstring;
static size_t x86_bsize, alpha_bsize, ppc_bsize, sparc_bsize;


static void do_report(Packet_t *packet, const unsigned int nopnbr,
                      const char *machine) 
{
        plugin_do_report((plugin_generic_t *) &plugin, packet, maybe_not_reliable,
                         "Stack overflow attack",
                         "%s stack overflow attempt, %u nop", machine, nopnbr);
}

static int cmp(const unsigned char *bstring, const unsigned int bsize, 
               const unsigned char *data, const size_t dsize) { 
        int i, ret;
        
        for ( i = 0; i < dsize; i += bsize ) {
                ret = memcmp(bstring, &data[i], bsize);
                if ( ret == 0 ) 
                        return 0;
        }
        
        return -1;
}

static void nopsearch_run(Packet_t *packet, int depth) {
        size_t size;
	int ret;
        unsigned char *p;

        assert(packet[depth].proto == p_data);

        p = packet[depth].p.pkt_data->data;
        size = packet[depth].p.pkt_data->size;

        if ( size >= alpha_bsize ) {
                ret = cmp(x86_bstring, x86_bsize, p, size);
                if ( ret == 0 ) {
                        do_report(packet, x86_bsize, "x86");
                        return;
                }
                
                ret = cmp(alpha_bstring, alpha_bsize, p, size);
                if ( ret == 0 ) {
                        do_report(packet, alpha_bsize, "alpha");
                        return;
                }
                
                ret = cmp(sparc_bstring, sparc_bsize, p, size);
                if ( ret == 0 ) {
                        do_report(packet, sparc_bsize, "sparc");
                        return;
                }
                
                ret = cmp(ppc_bstring, ppc_bsize, p, size);
                if ( ret == 0 ) {
                        do_report(packet, ppc_bsize, "ppc");
                        return;
                }
        } else if ( size >= x86_bsize ) {
                ret = cmp(x86_bstring, x86_bsize, p, size);
                if ( ret == 0 )
                        do_report(packet, x86_bsize, "x86");
        }
}


int plugin_init(unsigned int id) {
        unsigned char x86_nop[] = { 0x90 };
        unsigned char alpha_nop[] = { 0x1f, 0x04, 0xff, 0x47 };
        unsigned char ppc_nop[] = { 0x60, 0x00, 0x00, 0x00 };
        unsigned char sparc_nop[] = { 0x01, 0x00, 0x00, 0x00 };
        
        alloc_bstring(&x86_bstring, &x86_bsize, x86_nop, sizeof(x86_nop));
        alloc_bstring(&alpha_bstring, &alpha_bsize, alpha_nop, sizeof(alpha_nop));
        alloc_bstring(&ppc_bstring, &ppc_bsize, ppc_nop, sizeof(ppc_nop));
        alloc_bstring(&sparc_bstring, &sparc_bsize, sparc_nop, sizeof(sparc_nop));

        plugin_set_name(&plugin, "NopSearch");
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "By counting the number of NOP in the data of a packet,\n"
                        "it is possible to probe if it contain a stack overflow...");

        plugin_set_running_func(&plugin, nopsearch_run);
        plugin_set_subscribtion(&plugin, subscribtion);
        
        return plugin_register((plugin_generic_t *) &plugin);
}







