/*****
*
* Copyright (C) 1998 - 2000 Vandoorselaere Yoann <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <time.h>
#include <errno.h>

#include "report.h"

#define plural(number) (number > 1) ? "s" : ""

static FILE *fd;
static char *logfile = NULL;

/*
 *
 */
static const char *switch_alert_kind(rkind_t kind) 
{
        switch(kind) {

        case maybe_faked:
                return "Could be faked";

        case maybe_not_reliable:
                return "May not be reliable";

        case normal:
                return "Should be ok";

        }

        return "";
}



/*
 *
 */
static void filemod_run(alert_t *alert, report_infos_t *rinfos) {
        int i;
        
        fprintf(fd, "\n\n*** %s", rinfos->date_start);

        if ( rinfos->date_end ) {
                fprintf(fd, " - %s\n", rinfos->date_end);
        } else
                fprintf(fd, "\n");
        
        fprintf(fd, "Plugin\t: %s\n", alert_from(alert));
        fprintf(fd, "Author\t: %s\n", alert_author(alert));
        fprintf(fd, "Contact\t: %s\n", alert_contact(alert));
        fprintf(fd, "description\t: %s\n", alert_desc(alert));
        fprintf(fd, "kind\t\t: %s\n", switch_alert_kind(alert_kind(alert)));
        fprintf(fd, "received\t: %d time%s\n", alert_count(alert), plural(alert_count(alert)));
        fprintf(fd, "message\t\t: %s\n\n", alert_message(alert));
        
        if ( rinfos->pktdump ) 
                for ( i = 0; rinfos->pktdump[i] != NULL; i++) 
                        fprintf(fd, "%s\n", rinfos->pktdump[i]);
        
        if ( rinfos->hexdump ) {
                fprintf(fd, "\nData hexadecimal dump follow :\n");
                
                for ( i = 0; rinfos->hexdump[i] != NULL; i++ ) 
                        fprintf(fd, "%s\n", rinfos->hexdump[i]);
        }
}



static void filemod_close(void) 
{
        fclose(fd);
}



static void print_help(const char *pname) 
{
        fprintf(stderr, "Usage for %s :\n", pname);
        fprintf(stderr, "\t -e --enable Enable this plugin.\n");
        fprintf(stderr, "\t -f --logfile (default=disabled) "
                "Path to the file where %s should do reporting.\n", pname);
}



static int check_opts(char *pname) 
{
        char **argv;
        config_t *cfg;        
        int c, argc, enable = 0;
        struct option opts[] = {
                { "enable", no_argument, NULL, 'e' },
                { "logfile", required_argument, NULL, 'f' },
                { "help", no_argument, NULL, 'h' },
                { 0, 0, 0, 0 },
        };

        plugin_get_opts(pname, &argc, &argv);
        
        while ( (c = getopt_long(argc, argv, "f:eh", opts, NULL)) != -1 ) {
                switch (c) {
                case 'e':
                        enable = 1;
                        break;
                        
                case 'f':
                        logfile = strdup(optarg);
                        break;

                case 'h':
                        print_help(pname);
                        break;
                        
                default:
                        return -1;
                }
        }

        if ( (! enable || ! logfile ) && (cfg = config_open(PRELUDE_REPORT_CONF)) ) {
                const char *str;

                if ( ! enable && (str = config_get(cfg, pname, "enable") )) {
                        if ( strcmp(str, "yes") == 0 )
                                enable = 1;
                } else
                        return -1;

                if ( ! logfile && (str = config_get(cfg, pname, "logfile") ))
                        logfile = strdup(str);
        }
        
        return (enable == 1 && logfile) ? 0 : -1;
}



int plugin_init(unsigned int id)
{
        int ret;
        static plugin_report_t plugin;
    
        plugin_set_name(&plugin, "FileMod");
        
        ret = check_opts(plugin.name);
        if ( ret < 0 )
                return -1;

        fd = fopen(logfile, "a");
        if ( ! fd ) {
                log(LOG_ERR, "error opening %s for appending.\n", logfile);
                return -1;
        }
        
        plugin_set_author(&plugin, "Yoann Vandoorselaere");
        plugin_set_contact(&plugin, "yoann@mandrakesoft.com");
        plugin_set_desc(&plugin, "Will log all alert to a file.");
        plugin_set_running_func(&plugin, filemod_run);
        plugin_set_closing_func(&plugin, filemod_close);
        
	plugin_register((plugin_generic_t *)&plugin);

        return 0;
}












