/*****
*
* Copyright (C) 1998 - 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <time.h>
#include <errno.h>
#include <string.h>
#include <assert.h>
#include <sys/time.h>

#include "packet.h"
#include "plugin-common.h"
#include "plugin-detect.h"
#include "detect-plugins-api.h"
#include "common.h"
#include "alert.h"
#include "alert-prv.h"
#include "timer.h"
#include "rqueue.h"
#include "recycler.h"


void packet_release(Packet_t *packet)
{
        int i;

        /*
         * It is a bug if packet_release is
         * called without a locked packet.
         */
        assert(RecyclerIsLocked(packet) > 0);

        /*
         * Release the main chunk of memory,
         * in fact we only use the refcount for this Chunk,
         * which may be locked > 1 time.
         */
        RecyclerReleaseChunk(packet);

        /*
         * Check the refcount, if it is 0, release the rest.
         */
        if ( RecyclerIsLocked(packet) == 0 ) 
                for ( i = 0; packet[i].proto != p_end; i++ )
                        RecyclerReleaseChunk(packet[i].p.ip);
        
}



void packet_lock(Packet_t *packet) 
{
        int i;

        /*
         * Only lock the rest of the packet
         * if the main packet array is not locked yet.
         */
        if ( RecyclerIsLocked(packet) == 0 ) 
                for (i = 0; packet[i].proto != p_end; i++ )
                        RecyclerLockChunk(packet[i].p.ip);

        /*
         * We lock the packet array everytime, so we have
         * a valid refcount.
         */
        RecyclerLockChunk(packet);
}




