/*****
*
* Copyright (C) 1998, 1999, 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <dlfcn.h>
#include <errno.h>
#include <limits.h> /* for NAME_MAX */
#include <sys/time.h>

#include "packet.h"

#include "list.h"
#include "plugin-common.h"
#include "plugin-common-prv.h"
#include "plugin-detect.h"
#include "detect-plugins.h"
#include "common.h"

unsigned int plugins_id_max = 0;

LIST_HEAD(__tcp_plugins);
LIST_HEAD(__udp_plugins);
LIST_HEAD(__ip_plugins);
LIST_HEAD(__ipencap_plugins);
LIST_HEAD(__data_plugins);
LIST_HEAD(__icmp_plugins);
LIST_HEAD(__igmp_plugins);
LIST_HEAD(__arp_plugins);
LIST_HEAD(__rarp_plugins);
LIST_HEAD(__ether_plugins);
LIST_HEAD(__all_plugins);



/*
 * Try to register plugin 'plugin' to protocol 'wanted_proto',
 * return -1 on error, 0 on success.
 */
static void register_to_proto(plugin_container_t *pc, proto_enum_t wanted_proto) 
{
        int i;
        static struct {
                char *proto;
                proto_enum_t type;
                struct list_head *plugin_list;
        } tbl[] = {
                { "IP", p_ip, &__ip_plugins },
                { "IPENCAP", p_ipencap, &__ipencap_plugins },
                { "TCP", p_tcp, &__tcp_plugins },
                { "UDP", p_udp, &__udp_plugins },
                { "ICMP", p_icmp, &__icmp_plugins },
                { "IGMP", p_igmp, &__igmp_plugins },
                { "ARP", p_arp, &__arp_plugins },
                { "RARP", p_rarp, &__rarp_plugins },
                { "ETHER", p_ether, &__ether_plugins },
                { "DATA", p_data, &__data_plugins },
                { "ALL", p_all, &__all_plugins },
                { NULL, 0, NULL }
        };
        
        for ( i = 0; tbl[i].proto != NULL; i++ )
                if ( tbl[i].type == wanted_proto ) {
                        log(LOG_INFO, "%s", tbl[i].proto);
                        ((plugin_detect_t *)pc->plugin)->id = plugins_id_max++;
                        plugin_register_for_use(pc, tbl[i].plugin_list, tbl[i].proto);
                        return;
                }

        log(LOG_ERR,"plugin %s has unknow type %d.\n", pc->plugin->name, wanted_proto);
}



/*
 * Subscribe plugins to it's specified protocol,
 * if the plugin need to subscribe to > 1 protocol :
 * we need to copy it.
 */
static int plugin_subscribe(plugin_container_t *pc)
{
	int i;
        plugin_detect_t *plugin = (plugin_detect_t *) pc->plugin;
        
        log(LOG_INFO, "\t\t%s subscribing to : \"[", plugin->name);
        
	for (i = 0; plugin->subscribtion[i].type != p_end; i++) {
                if ( i > 0 ) 
                        log(LOG_INFO, ",");
                register_to_proto(pc, plugin->subscribtion[i].type);
        }

        log(LOG_INFO, "]\".\n");

        return 0;
}


/*
 * Start all plugins of kind 'list'.
 */
extern int depth;

void detect_plugins_run(Packet_t *packet, struct list_head *list, char *proto)
{
        struct list_head *tmp;
        plugin_container_t *pc;

        list_for_each(tmp, list) {
                pc = list_entry(tmp, plugin_container_t, ext_list);
                plugin_run(pc, plugin_detect_t, packet, depth);
        }
}



/*
 * Open the plugin directory (dirname),
 * and try to load all plugins located int it.
 */
int detect_plugins_init(const char *dirname)
{
        int ret;

        ret = plugin_load_from_dir(dirname, plugin_subscribe);
        if ( ret < 0 ) {
                log(LOG_ERR, "couldn't load plugin subsystem.\n");
                return -1;
        }

        return plugins_id_max;
}


