/*****
*
* Copyright (C) 1998, 1999, 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <stdlib.h>
#include <signal.h>

#include "pconfig.h"
#include "daemonize.h"
#include "capture.h"
#include "packet.h"
#include "auth.h"
#include "common.h"
#include "list.h"
#include "plugin-common.h"
#include "plugin-common-prv.h"
#include "detect-plugins.h"
#include "alert.h"
#include "alert-prv.h"
#include "rsend.h"
#include "async-write.h"

Pconfig_t config;


static void cleanup(void) 
{
        capture_stats();
        capture_stop();
        plugins_print_stats();        
        async_exit();
        if (config.pidfile)
                unlink(config.pidfile);
}



static void sighandler(int sig)
{
	fprintf(stderr, "\n\nCaught signal %d.\n", sig);

        signal(sig, SIG_DFL);

        cleanup();

        
        /*
         * Currently, exit with status 2 for gprof analysis.
         */
#if 0
        raise(sig);
#endif    
        exit(2);
}

int main(int argc, char **argv)
{
        int ret;
        
        if (getuid() != 0 || geteuid() != 0) {
                fprintf(stderr, "\n\nPrelude need root access in order to run.\n\n");
		exit(1);
	}

        if ( pconfig_set(argc, argv) < 0 )
                exit(1);

        log(LOG_INFO, "\nPrelude, (c) 1998 - 2000 Vandoorselaere Yoann.\n"
            "Developed under the GPL license.\n\n");

        /*
         * Detections plugins initialization
         */
        do_init(detect_plugins_init(DETECT_PLUGIN_DIR),
                "Initializing detections plugins.");

        /*
         * Creation of signal handler.
         */
	signal(SIGTERM, sighandler);
	signal(SIGINT, sighandler);
	signal(SIGSEGV, sighandler);

        
        /*
         * Start prelude as a daemon if asked.
         */
        if ( config.daemonize == 1 )
                do_init(daemon_start(config.pidfile),
                        "Starting Prelude as a daemon.");
        
        /*
         * Connect to the report server, for future report sending.
         */
        do_init(rsend_init(),
                "Initializing connection to report server.");
        
        /*
         * Prelude capture begin here.
         */
        do_init(1, "Initializing packet capture");
        ret = capture_start();
        
        cleanup();

        exit(ret);
}




