/*****
*
* Copyright (C) 2000 Yoann Vandoorselaere <yoann@mandrakesoft.com>
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by 
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/uio.h>

#ifdef HAVE_SENDFILE
#include <sys/sendfile.h>
#endif

#ifdef HAVE_XDR
#include "pconfig.h"
extern Pconfig_t config;
#endif

#include <fcntl.h>

#include "common.h"
#include "packet.h"
#include "alert.h"
#include "alert-prv.h"
#include "alert-common.h"
#include "rwrite.h"
#include "rsave.h"
#include "write-func.h"


#define RFILE "/var/lib/prelude/report"


static int fd = -1;



/*
 *
 */
static int setup_fd(void) 
{        
        fd = open(RFILE, O_RDWR|O_CREAT|O_APPEND, S_IRUSR|S_IWUSR);
        if ( fd < 0 ) {
                log(LOG_ERR,"opening %s for appending.\n", RFILE);
                return -1;
        }         

        return 0;
}



/*
 * backup a report.
 */
int backup_report(alert_t *alert) 
{
        int ret;

        if ( fd < 0 ) {
                ret = setup_fd();
                if ( ret < 0 )
                        return -1;
        }

        ret = writev_raw_report(fd, alert);
        if ( ret < 0 )
                return -1;

        return 0;
}



#ifdef HAVE_SENDFILE

static int sendfile_send(int sock)
{
        int ret;
        off_t off = 0;
        struct stat st;
        
        ret = fstat(fd, &st);
        if ( ret < 0 ) {
                log(LOG_ERR, "couldn't stat backup file descriptor.\n");
                return -1;
        }

        while ( 1 ) {
                ret = sendfile(sock, fd, &off, st.st_size);
                if ( off > 0 && off < st.st_size ) 
                        continue;
                
                break;
        }
        
        if ( off != st.st_size )
                log(LOG_ERR, "sendfile couldn't send %d bytes.\n",
                    (size_t) st.st_size);
        
        return ret;
}

#endif



/*
 *
 */
static int alert_send(int sock) 
{
        int ret;
        alert_t alert;
        Packet_t packet[MAX_PKTDEPTH];
        
        alert_packet(&alert) = packet;
        
        while ( alert_read(fd, &alert, read) > 0 ) {
                ret = rwrite_write(&alert, 0);
                if ( ret < 0 )
                        break;
                
                alert_free(&alert, 1);
        }
        
        return 0;   
}




/*
 * return the number of character read,
 * 0 for end of file, -1 for error.
 */
int backout_report(int sock) 
{
        int ret;

        if ( fd < 0 )
                return -1;
        
        log(LOG_INFO, "Flushing backuped report : ");

        ret = lseek(fd, 0, SEEK_SET);
        if ( ret < 0 ) {
                log(LOG_ERR, "couldn't seek to the begining of the file.\n");
                return -1;
        }
        
#ifdef HAVE_SENDFILE
        if ( ! config.use_ssl && ! config.use_xdr ) 
                ret = sendfile_send(sock);
        else
#endif
                ret = alert_send(sock);

        close(fd);
        fd = -1;
        unlink(RFILE);
        
        log(LOG_INFO, "%s.\n", (ret < 0) ? "Failed" : "Done");
        
        return ret;
}













