#ifdef HAVE_SSL

/*****
*
* Copyright (C) 2001 Jeremie Brebec / Toussaint Mathieu
* All Rights Reserved
*
* This file is part of the Prelude program.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2, or (at your option)
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; see the file COPYING.  If not, write to
* the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*
*****/

#include <stdlib.h>
#include <stdio.h>
#include <memory.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <openssl/x509.h>
#include <openssl/x509v3.h>
#include <openssl/pem.h>
#include <openssl/bio.h>

#include "ssl_gencrypto.h"
#include "ssl_registration_msg.h"
#include "ssl_config.h"
#include "ssl.h"
#include "pconfig.h"

#define ACKMSGLEN ACKLENGTH + SHA_DIGEST_LENGTH + HEADLENGTH + PADMAXSIZE


extern Pconfig_t config;


int ssl_add_certificate(void)
{

	int sd, err;
	struct sockaddr_in sa;
	des_key_schedule skey1;
	des_key_schedule skey2;
	X509 *x509ss;


	BUF_MEM ackbuf;
	char buf[BUFMAXSIZE];
	char cert[BUFMAXSIZE];
	int len, certlen;

	ssl_read_config(PRELUDE_CONF);

	if (des_generate_2key(&skey1, &skey2, 0) != 0) {
		fprintf(stderr, "Problem making one shot password\n");
		fprintf(stderr, "\nRegistration failed\n");
		return EXIT_FAILURE;
	}

        fprintf(stderr, "\nNow please start a report server with the -w flag.\n"
                "press enter when done.\n");
        
        fgets(buf, sizeof(buf), stdin);
        
	fprintf(stderr, "connecting to Prelude-Report (%s:%d)...\n",
                config.addr, config.port);
	/* Create a socket and connect to server */

	sd = socket(AF_INET, SOCK_STREAM, 0);
	if (sd == -1) {
		perror("socket");
		fprintf(stderr, "\nRegistration failed\n");
		return EXIT_FAILURE;
	}

	memset(&sa, '\0', sizeof(sa));
	sa.sin_family = AF_INET;
	sa.sin_addr.s_addr = inet_addr(config.addr);
	sa.sin_port = htons(config.port);

	err = connect(sd, (struct sockaddr *) &sa, sizeof(sa));
	if (err == -1) {
		perror("connect");
		fprintf(stderr, "\nRegistration failed\n");
		return EXIT_FAILURE;
	}

	fprintf(stderr, "Connected\n");
	/* dialogue */
	x509ss = ssl_gen_crypto(ssl_get_days(), ssl_get_key_length(),
                                ssl_get_cert_filename(PRELUDE_KEY), config.ssl_key_crypt);
	if (x509ss == NULL) {
		fprintf(stderr, "\nRegistration failed\n");
		return EXIT_FAILURE;
	}

	/* prepare request message */
	len = x509_to_msg(x509ss, buf, BUFMAXSIZE, &skey1, &skey2);
	if (len < 0) {
		fprintf(stderr, "Error making request\n");
		fprintf(stderr, "\nRegistration failed\n");
		return EXIT_FAILURE;
	}

	/* send own certificate */
	len = send(sd, buf, len, 0);

	/* receive Prelude-Report certificate */
	len = recv(sd, buf, BUFMAXSIZE, 0);
	if (len <= 0) {
		perror("recv");
		fprintf(stderr, "\nRegistration failed\n");
		close(sd);
		return EXIT_FAILURE;
	}

	certlen =
	    analyse_install_msg(buf, len, cert, BUFMAXSIZE, &skey1,
				&skey2);
	if (certlen < 0) {
		fprintf(stderr, "Bad message received\n");
		fprintf(stderr, "\nRegistration failed\n");
		close(sd);
		return EXIT_FAILURE;
	}


	fprintf(stderr, "writing Prelude-Report certificate\n");

	/* save Prelude-Report certificate */
	if (!save_cert(ssl_get_cert_filename(REPORT_CERT), cert, certlen)) {
		fprintf(stderr,
			"error writing Prelude-Report Certificate to\n");
		close(sd);
		return EXIT_FAILURE;
	}

	/* send ack */
	ackbuf.length = ACKLENGTH;
	ackbuf.data = ACK;
	ackbuf.max = ACKLENGTH;

	len = build_install_msg(&ackbuf, buf, ACKMSGLEN, &skey1, &skey2);
	if (len <= 0) {
		fprintf(stderr, "Error building message\n");
		fprintf(stderr, "\nRegistration failed\n");
		close(sd);
		return EXIT_FAILURE;
	}
	send(sd, buf, len, 0);

	fprintf(stderr, "\nRegistration completed\n");
	fprintf(stderr,
		"\nPrelude is now ready to send intrusion reports\n");
	close(sd);

	return EXIT_SUCCESS;
}

#endif
